@@ -2,7 +2,7 @@
 #'
 #'\code{PPforest} implements a random forest using projection pursuit trees algorithm (based on PPtreeViz package).
 #' @usage PPforest(data, class, std = TRUE, size.tr, m, PPmethod, size.p,
-#'  lambda = .1, parallel = FALSE, cores = 2)
+#'  lambda = .1, parallel = FALSE, cores = 2, rule = 1)
 #' @param data Data frame with the complete data set.
 #' @param class A character with the name of the class variable.
 #' @param std if TRUE standardize the data set, needed to compute global importance measure. 
@@ -13,6 +13,7 @@
 #' @param lambda penalty parameter in PDA index and is between 0 to 1 . If \code{lambda = 0}, no penalty parameter is added and the PDA index is the same as LDA index. If \code{lambda = 1} all variables are treated as uncorrelated. The default value is \code{lambda = 0.1}.
 #' @param parallel logical condition, if it is TRUE then  parallelize the function
 #' @param cores number of cores used in the parallelization
+#' @param rule split rule 1: mean of two group means 2: weighted mean of two group means - weight with group size 3: weighted mean of two group means - weight with group sd 4: weighted mean of two group means - weight with group se 5: mean of two group medians 6: weighted mean of two group medians - weight with group size 7: weighted mean of two group median - weight with group IQR 8: weighted mean of two group median - weight with group IQR and size
 #' @return An object of class \code{PPforest} with components.
 #' \item{prediction.training}{predicted values for training data set.}
 #' \item{training.error}{error of the training data set.}
@@ -31,21 +32,27 @@
 #' \item{call}{the original call to \code{PPforest}.}
 #' \item{train}{is the training data based on \code{size.tr} sample proportion}
 #' \item{test}{is the test data based on \code{1-size.tr} sample proportion}
+#'@references Natalia da Silva, Dianne Cook & Eun-Kyung Lee (2021) 
+#'A Projection Pursuit Forest Algorithm for Supervised Classification, 
+#'Journal of Computational and Graphical Statistics,
+#' DOI: 10.1080/10618600.2020.1870480
 #' @export
 #' @examples
 #' #crab example with all the observations used as training
 #' 
 #'pprf.crab <- PPforest(data = crab, class = 'Type',
-#'  std = FALSE, size.tr = 1, m = 200, size.p = .5, PPmethod = 'LDA' , parallel = TRUE, cores = 2)
+#'  std = FALSE, size.tr = 1, m = 200, size.p = .5, 
+#'  PPmethod = 'LDA' , parallel = TRUE, cores = 2, rule=1)
 #' pprf.crab
 #' 
 PPforest <- function(data, class, std = TRUE, size.tr = 2/3, m = 500, PPmethod, size.p, lambda = 0.1, 
-    parallel = FALSE, cores = 2) {
+    parallel = FALSE, cores = 2, rule = 1) {
     
     Var1 <- NULL
     tree <- NULL
     pred <- NULL
     id <- NULL
+
     if (std) {
         dataux <- data %>% dplyr::select(-(!!class)) %>% apply(2, FUN = scale) %>% dplyr::as_data_frame()
         data <- data.frame(data[, class], dataux)
@@ -66,7 +73,7 @@ PPforest <- function(data, class, std = TRUE, size.tr = 2/3, m = 500, PPmethod,
     output <- lapply(outputaux, function(x) x[[1]])
     
     data.b <- lapply(outputaux, function(x) x[[2]])
-    pred.tr <- trees_pred(outputaux, xnew = dplyr::select(train, -(!!class)), parallel, cores = cores)
+    pred.tr <- trees_pred(outputaux, xnew = dplyr::select(train, -(!!class)), parallel, cores = cores, rule = rule)
     
     expand.grid.ef <- function(seq1, seq2) {
         data.frame(a = rep.int(seq1, length(seq2)), b = rep.int(seq2, rep.int(length(seq1), 
@@ -74,14 +81,13 @@ PPforest <- function(data, class, std = TRUE, size.tr = 2/3, m = 500, PPmethod,
     }
     pos <- expand.grid.ef(1:dim(train)[1], 1:dim(train)[1])
     
-    proximity <- proximi((pred.tr[[1]]), m)
+    proximity <- proximi((pred.tr$predtree), m)
     
     index <- oobindex(data.b, m)
     
     oob.obs <- oobobs(index)
     
-    
-    mvote.oob <- mvoteoob(pred.tr[[1]], oob.obs)
+    mvote.oob <- mvoteoob(pred.tr$predtree, oob.obs)
     oob.pred <- mvote.oob[, length(unique(clnum)) + 1]
     
     votes <- mvote.oob[, -(length(unique(clnum)) + 1)]
@@ -92,14 +98,15 @@ PPforest <- function(data, class, std = TRUE, size.tr = 2/3, m = 500, PPmethod,
     oob.error <- 1 - sum(diag(table(oob.pred, unlist(train[, class]))))/length(unlist(train[, 
         class]))
     
-    oob.err.tree <- ooberrortree(pred.tr[[1]], oob.obs, as.numeric(as.factor(unlist(train[, 
+    oob.err.tree <- ooberrortree(pred.tr$predtree, oob.obs, as.numeric(as.factor(unlist(train[, 
         class]))), m)
     
-    error.tr <- 1 - sum(as.numeric(as.factor(unlist(train[, class]))) == pred.tr[[2]])/length(pred.tr[[2]])
+  
+    error.tr <- 1 - sum(as.numeric(as.factor(unlist(train[, class]))) == pred.tr$predforest)/length(pred.tr$predforest)
     test <- data[-tr.index, ] %>% dplyr::select(-(!!class)) %>% dplyr::filter_()
     
     if (dim(test)[1] != 0) {
-        pred.test <- trees_pred(outputaux, xnew = test, parallel, cores = cores)
+        pred.test <- trees_pred(outputaux, xnew = test, parallel, cores = cores, rule = rule)
         error.test <- 1 - sum(as.numeric(as.factor(data[-tr.index, class])) == pred.test[[2]])/length(pred.test[[2]])
         pred.test = as.factor(pred.test[[2]])
         levels(pred.test) <- levels(unlist(train[, class]))
@@ -116,7 +123,7 @@ PPforest <- function(data, class, std = TRUE, size.tr = 2/3, m = 500, PPmethod,
         levels(oob.pred) <- levels(as.factor(unlist(train[, class])))
     }
     
-    prediction.training <- as.factor(pred.tr[[2]])
+    prediction.training <- as.factor(pred.tr$predforest)
     if (is.factor(unlist(train[, class]))) {
         levels(prediction.training) <- levels(unlist(train[, class]))
     } else {
@@ -129,12 +136,13 @@ PPforest <- function(data, class, std = TRUE, size.tr = 2/3, m = 500, PPmethod,
     class.error <- 1 - diag(tab.tr)/((stats::addmargins(tab.tr, 2))[, "Sum"])
     confusion <- cbind(tab.tr, class.error = round(class.error, 2))
     
-    results <- list(predicting.training = prediction.training, training.error = error.tr, prediction.test = pred.test, 
-        error.test = error.test, oob.error.forest = oob.error, oob.error.tree = oob.err.tree, 
-        boot.samp = data.b, output.trees = output, proximity = proximity, votes = vote.matrix.prop, 
-        prediction.oob = oob.pred, n.tree = m, n.var = var.sel, type = "Classification", confusion = confusion, 
-        call = match.call(), train = train, test = test, vote.mat = pred.tr[[1]], class.var = class, 
+    results <- list(predicting.training = prediction.training, training.error = error.tr, prediction.test = pred.test,
+        error.test = error.test, oob.error.forest = oob.error, oob.error.tree = oob.err.tree,
+        boot.samp = data.b, output.trees = output, proximity = proximity, votes = vote.matrix.prop,
+        prediction.oob = oob.pred, n.tree = m, n.var = var.sel, type = "Classification", confusion = confusion,
+        call = match.call(), train = train, test = test, vote.mat = pred.tr$predtree, class.var = class,
         oob.obs = oob.obs)
+  
     
     class(results) <- "PPforest"
     
